siboehm/SGEMM_CUDA (19 files):
siboehm_sgemm.cu, siboehm_runner.cu, siboehm_runner.cuh, siboehm_kernels.cuh
siboehm_cuBLAS_sgemm.cu, siboehm_simplest_kernel.cu, siboehm_CMakeLists.txt
siboehm_{1_naive..12_kernel_double_buffering}.cuh
wangzyon/NVIDIA_SGEMM_PRACTICE (12 files):
wangzyon_sgemm.cu, wangzyon_utils.cu, wangzyon_utils.cuh, wangzyon_kernel.cuh
wangzyon_CMakeLists.txt, wangzyon_kernel_{1..7}.cuh
edtallison/sgemm-cuda (19 files):
edtallison_sgemm.cu, edtallison_runner.cu, edtallison_runner.cuh
edtallison_kernels.cuh, edtallison_cuBLAS_sgemm.cu, edtallison_simplest_kernel.cu
edtallison_CMakeLists.txt, edtallison_{01_naive..12_kernel_double_buffering}.cuh
cat_files/ total: 25 → 75 files
38 lines
1.4 KiB
Plaintext
38 lines
1.4 KiB
Plaintext
# pragma once
|
|
|
|
#include <cstdio>
|
|
#include <cstdlib>
|
|
#include <cublas_v2.h>
|
|
#include <cuda_runtime.h>
|
|
|
|
/*
|
|
|
|
Matrix sizes:
|
|
MxK * KxN = MxN
|
|
|
|
*/
|
|
|
|
__global__ void sgemm_naive(
|
|
int M, int N, int K, // sizes
|
|
float alpha, const float *A, const float *B, float beta, float *C // pointers used to point to matrices
|
|
) {
|
|
// compute position in C that this thread is responsible for
|
|
// "which block" * "width of block" to get to start of block + "which thread"
|
|
const uint x = blockIdx.x * blockDim.x + threadIdx.x; // "which row?" (inverted from graphical intuition, confusingly)
|
|
const uint y = blockIdx.y * blockDim.y + threadIdx.y; // "which column?"
|
|
|
|
// if M or N are not multiples of 32, there will be "extra"/"remainder" threads on the last block in x/y.
|
|
// we don't want those leftover threads to do anything (tile quantisation)
|
|
if (x < M && y < N) {
|
|
float tmp = 0.0;
|
|
for (int i = 0; i < K; ++i) { // K is the size of the row in A, col in B i.e. the dot product
|
|
// A: x * K gives the start of relevant row, i enumerates across the row (col by col)
|
|
// B: y gives the relevant column, i * N enumerates down the column, (row by row)
|
|
tmp += A[x * K + i] * B[i * N + y];
|
|
}
|
|
// C = alpha*(A@B) + beta*C
|
|
// x * N takes to start of relevant row, y moves across to the relevant column
|
|
C[x * N + y] = alpha * tmp + beta * C[x * N + y];
|
|
}
|
|
}
|